// menuchange.cpp
// by Glenn G. Chappell
// September 2003
//
// For CS 381
// Demo of changing GLUT menu

#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff, includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character

// Square
bool redsquare = true;        // false: blue square, true: red square


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT);

   if (redsquare)
      glColor3d(0.9, 0.1, 0.1);
   else
      glColor3d(0.1, 0.1, 0.9);

   glBegin(GL_QUADS);  // Draw a square
      glVertex2d(-0.7, -0.7);
      glVertex2d( 0.7, -0.7);
      glVertex2d( 0.7,  0.7);
      glVertex2d(-0.7,  0.7);
   glEnd();

   glutSwapBuffers();
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   glViewport(0, 0, w, h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // We set up coordinate system so that aspect ratios are always correct,
   //  and the region from -1..1 in x & y always just fits in the viewport.
   if (w > h)
   {
      gluOrtho2D(-double(w)/h, double(w)/h, -1., 1.);
   }
   else
   {
      gluOrtho2D( -1., 1., -double(h)/w, double(h)/w);
   }
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   default:      // Any other ASCII key refreshes display
      glutPostRedisplay();
      break;
   }
}


// menu_entry
// Constructs & returns currnet text for color-toggle menu entry.
string menu_entry()
{
   string entry = "Change square color to ";
   if (redsquare)
      entry += "blue";
   else
      entry += "red";

   return entry;
}


// handle_main_menu
// Menu handling callback function for our menu
void handle_main_menu(int value)
{

   switch (value)
   {
   case 1:   // Toggle square color
      redsquare = !redsquare;
      glutChangeToMenuEntry(1, menu_entry().c_str(), 1);
         // The two 1's above mean different things!
         // The first is which entry we are changing (numbers start with 1).
         // The second is the parameter that is passed to the handler function
         //  when the user selects this item.
      glutPostRedisplay();
      break;
   case 2:   // Quit
      exit(0);
      break;
   case 999: // Other menu items do nothing
      break;
   }
}


// make_main_menu
// Creates the main menu, using function handle_main_menu as the handler callback
void make_main_menu()
{
   glutCreateMenu(handle_main_menu);
   glutAddMenuEntry(menu_entry().c_str(), 1);
   glutAddMenuEntry("Quit", 2);
   glutAddMenuEntry("------------------------", 999);
   glutAddMenuEntry("Or press Escape to quit", 999);
   glutAttachMenu(GLUT_RIGHT_BUTTON);
}


// init
// General initialization
// Called by main.
void init() 
{
   make_main_menu();

   glClearColor(1., 1., 1., 0.);
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - Changing Menu Demo - Right mouse for menu");

   // Initialize & register callbacks
   init();
   glutDisplayFunc(display);
   glutIdleFunc(idle);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);

   // Do something
   glutMainLoop();

   return 0;
}
